<?php
/**
 * Hermes Base Class.
 *
 * $Horde: hermes/lib/Hermes.php,v 1.34 2004/11/23 21:53:49 chuck Exp $
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.34 $
 * @package Hermes
 */
class Hermes {

    function &getDriver()
    {
        global $conf;

        require_once HERMES_BASE . '/lib/Driver.php';
        return Hermes_Driver::singleton();
    }

    function listClients()
    {
        static $clients;

        if (is_null($clients)) {
            $args = array('addresses' => array(''),
                          'fields' => array('name')
                          );
            $result = $GLOBALS['registry']->call('clients/searchClients', $args);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
            $clients = array();
            if (!empty($result)) {
                $result = $result[''];
                foreach ($result as $client) {
                    $clients[$client['id']] = $client['name'];
                }
            }

            asort($clients, SORT_STRING);
        }

        return $clients;
    }

    function getDayOfWeek($timestamp)
    {
        // Return 0-6, indicating the position of $timestamp in the
        // period.
        $dow = 7 + date('w', $timestamp) - $GLOBALS['prefs']->getValue('start_of_week');
        return ($dow % 7);
    }

    /**
     * Build Hermes' list of menu items.
     *
     * @access public
     */
    function getMenu($returnType = 'object')
    {
        global $registry, $conf, $browser;

        require_once 'Horde/Menu.php';

        $menu = &new Menu();
        $menu->add(Horde::applicationUrl('time.php'), _("My _Time"), 'hermes.png', null, null, null, basename($_SERVER['PHP_SELF']) == 'index.php' ? 'current' : null);
        $menu->add(Horde::applicationUrl('entry.php'), _("_Enter Time"), 'hermes.png');
        $menu->add(Horde::applicationUrl('search.php'), _("_Search"), 'search.png', $registry->getImageDir('horde'));

        /* Print. */
        if ($conf['menu']['print'] && isset($print_link) && $browser->hasFeature('javascript')) {
            $menu->add("javascript:popup('$print_link'); return false;", _("_Print"), 'print.png', $registry->getImageDir('horde'));
        }

        /* Administration. */
        if (Auth::isAdmin()) {
            $menu->add(Horde::applicationUrl('admin.php'), _("_Admin"), 'hermes.png');
        }

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

    function canEditTimeslice($id)
    {
        global $hermes;

        $hours = $hermes->getHours(array('id' => $id));
        if (!is_array($hours) || count($hours) != 1) {
            return false;
        }
        $slice = $hours[0];

        // We can edit our own time if it hasn't been submitted.
        if ($slice['employee'] == Auth::getAuth() && !$slice['submitted']) {
            return true;
        }

        if ($GLOBALS['perms']->hasPermission('hermes:review', Auth::getAuth(), PERMS_EDIT)) {
            return true;
        }

        return false;
    }

    /**
     * Rewrite an hours array into a format useable by Horde_Data::
     *
     * @param array $hours          This is an array of the results from
     *                              $hermes->getHours().
     * @return array an array suitable for Horde_Data::
     */
    function makeExportHours($hours)
    {
        if (is_null($hours)) {
            return null;
        }
        require_once 'Horde/Identity.php';

        $namecache = array();
        for ($i = 0; $i < count($hours); $i++) {
            $timeentry = &$hours[$i];
            $timeentry['item'] = $timeentry['_type_name'];
            $emp = &$timeentry['employee'];
            if (isset($namecache[$emp])) {
                $emp = $namecache[$emp];
            } else {
                $ident = &Identity::singleton('none', $emp);
                $fullname = $ident->getValue('fullname');
                if ($fullname) {
                    $namecache[$emp] = $emp = $fullname;
                } else {
                    $namecache[$emp] = $emp;
                }
            }
        }

        return $hours;
    }

    /**
     * Get form control type for users.  What type of control we use depends
     * on whether the auth driver has list capability.
     *
     * @param optional string $enumtype         The type to return if we have
     *                                          list capability (should be
     *                                          either 'enum' or 'multienum').
     * @return a two-element array of the type and the type's parameters.
     */
    function getEmployeesType($enumtype = 'multienum')
    {
        require_once 'Horde/Identity.php';
        $auth = &Auth::singleton($GLOBALS['conf']['auth']['driver']);
        if (!$auth->capabilities['list']) {
            return array('text', array());
        }
        $users = $auth->listUsers();
        if (is_a($users, 'PEAR_Error')) {
            return array('invalid',
                         array(sprintf(_("An error occurred listing users: %s"),
                                       $users->getMessage())));
        }

        foreach ($users as $user) {
            $identity = &Identity::singleton('none', $user);
            $employees[$user] = $identity->getValue('fullname');
        }
        return array($enumtype, array($employees));
    }

}
